package com.tzld.piaoquan.recommend.server.service;

import com.google.common.reflect.TypeToken;
import com.tzld.piaoquan.recommend.feature.model.feature.FeatureKeyProto;
import com.tzld.piaoquan.recommend.server.remote.FeatureV2RemoteService;
import com.tzld.piaoquan.recommend.server.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author dyp
 */
@Service
@Slf4j
public class FeatureService {

    @Autowired
    private FeatureV2RemoteService remoteService;

    /**
     * @return k1:视频、k2:表、k3:特征、v:特征值
     */
    public Map<String, Map<String, Map<String, String>>> getFeature(List<String> vidList, String appType,
                                                                    String province) {

        List<FeatureKeyProto> protos = new ArrayList<>();

        for (String vid : vidList) {
            // TODO 补充其他特征
            // vid
            protos.add(genWithVid("alg_vid_feature_all_exp", vid));
            protos.add(genWithVid("alg_vid_feature_all_return", vid));
            protos.add(genWithVid("alg_vid_feature_all_share", vid));
            protos.add(genWithVid("alg_vid_feature_exp2share", vid));
            protos.add(genWithVid("alg_vid_feature_feed_flow_exp", vid));
            protos.add(genWithVid("alg_vid_feature_feed_flow_root_return", vid));
            protos.add(genWithVid("alg_vid_feature_feed_flow_root_share", vid));
            protos.add(genWithVid("alg_vid_feature_feed_noflow_exp", vid));
            protos.add(genWithVid("alg_vid_feature_feed_noflow_root_return", vid));
            protos.add(genWithVid("alg_vid_feature_feed_noflow_root_share", vid));
            protos.add(genWithVid("alg_vid_feature_feed_play", vid));
            protos.add(genWithVid("alg_vid_feature_head_play", vid));
            protos.add(genWithVid("alg_vid_feature_share2return", vid));
            // vid + apptype
            protos.add(genWithVidAndAppType("alg_vid_feature_feed_apptype_exp", vid, appType));
            protos.add(genWithVidAndAppType("alg_vid_feature_feed_apptype_root_return", vid, appType));
            protos.add(genWithVidAndAppType("alg_vid_feature_feed_apptype_root_share", vid, appType));

            // vid + province
            protos.add(genWithVidAndProvince("alg_vid_feature_feed_province_exp", vid, province));
            protos.add(genWithVidAndProvince("alg_vid_feature_feed_province_root_return", vid, province));
            protos.add(genWithVidAndProvince("alg_vid_feature_feed_province_root_share", vid, province));

            // vid + headvid
            protos.add(genWithVidAndHeadVid("", vid, headVid));
        }


        // user
        protos.add(genWithVidAndProvince("", mid));
        protos.add(genWithVidAndProvince("", mid));
        protos.add(genWithVidAndProvince("", mid));
        protos.add(genWithVidAndProvince("", mid));
        protos.add(genWithVidAndProvince("", mid));


        Map<String, String> result = remoteService.getFeature(protos);

        Map<String, Map<String, Map<String, String>>> data = new HashMap<>();

        result.entrySet().forEach(e -> {

            Map<String, String> colMap = JSONUtils.fromJson(e.getValue(), new TypeToken<Map<String, String>>() {
            }, Collections.emptyMap());
            String feature = colMap.get("feature");

            String[] uk = StringUtils.split(e.getKey(), ":");
            String vid = uk[1];
            String table = uk[0];
            Map<String, Map<String, String>> tableFeatureMap = data.getOrDefault(vid, new HashMap<>());
            tableFeatureMap.put(table, JSONUtils.fromJson(feature, new TypeToken<Map<String, String>>() {
            }, Collections.emptyMap()));
            data.put(vid, tableFeatureMap);
        });

        return data;
    }

    private final String ukFormat = "%s:%s";

    private FeatureKeyProto genWithVid(String table, String vid) {
        return FeatureKeyProto.newBuilder()
                .setUniqueKey(String.format(ukFormat, table, vid))
                .setTableName(table)
                .putFieldValue("vid", vid)
                .build();
    }

    private FeatureKeyProto genWithVidAndAppType(String table, String vid, String appType) {
        return FeatureKeyProto.newBuilder()
                .setUniqueKey(String.format(ukFormat, table, vid))
                .setTableName(table)
                .putFieldValue("vid", vid)
                .putFieldValue("apptype", appType)
                .build();
    }

    private FeatureKeyProto genWithVidAndProvince(String table, String vid, String province) {
        return FeatureKeyProto.newBuilder()
                .setUniqueKey(String.format(ukFormat, table, vid))
                .setTableName(table)
                .putFieldValue("vid", vid)
                .putFieldValue("province", province)
                .build();
    }

    private FeatureKeyProto genWithVidAndHeadVid(String table, String vid, String headVid) {
        return FeatureKeyProto.newBuilder()
                .setUniqueKey(String.format(ukFormat, table, vid))
                .setTableName(table)
                .putFieldValue("vid", vid)
                .putFieldValue("headVid", )
                .build();
    }

    private FeatureKeyProto genWithMid(String table, String mid) {
        return FeatureKeyProto.newBuilder()
                .setUniqueKey(String.format(ukFormat, table, mid))
                .setTableName(table)
                .putFieldValue("mid", )
                .build();
    }


}
